import java.io.*;

// XDemoObject must be public because the deserialization must create
// an instance of that class.
public class XDemoObject implements java.io.XSerializable {
  // java.io.XSerializable is a subinterface of
  // java.lang.Serializable.  to make your class amenable for the
  // better serialization, it has to implement this interface.  In
  // contrast to the basic serialization, we do have to provide both
  // an objectOut() and an objectIn() routine. The objectOut() routine
  // copies the contents of the object immediately into the stream's
  // communication buffer. The objectIn() routine reads from that
  // buffer. In future we plan to provide a tool that automatically
  // generates these routines from the bytecode.

  int   d0; // 4 bytes
  float d1; // 4 bytes
  char  d2; // 2 bytes
  short d3; // 2 bytes

  private final static int SIZE = 12;

  // XDemoObject needs a public default constructor, so
  // deserialization can create an uninitialized instance.
  public XDemoObject() {}

  public void objectOut(java.io.XObjectOutputStream s) throws java.io.IOException {
    int    bufcnt;

    // get the serialization buffer
    byte[] buf    = s.buf;  

    s.reserve(SIZE); 
    // reserve SIZE bytes in the serialization buffer determine the
    // number of bytes that is necessary to cover all instance
    // variables If more than XObjectInputStream.REQUEST_MAX (which is
    // currently 1024) bytes are necessary, the instance variables
    // have to be processed in groups of at most 1024 bytes.

    bufcnt = s.bufcnt;
    // bufcnt points to the current position in the communication
    // buffer. This index is advanced whenever an instance variable is
    // written to the buffer.
    
    bufcnt = java.io.UtilsIO.insert(buf, bufcnt, d0);
    bufcnt = java.io.UtilsIO.insert(buf, bufcnt, d1);
    bufcnt = java.io.UtilsIO.insert(buf, bufcnt, d2);
    bufcnt = java.io.UtilsIO.insert(buf, bufcnt, d3);

    // When done with all instance variables (or with a group of at
    // most 1024 bytes) call deliver();
    s.deliver(SIZE);

    // Our stream implementation guarantees, that whatever size you
    // write in one chunk, can later be read in a chunk of the same
    // size. Hence reading will be much easier, because you don't have
    // to deal with end-of-buffer conditions.
  }

  public void objectIn(java.io.XObjectInputStream s) throws java.io.IOException {
    int    bufcnt;
    byte[] buf    = s.buf;
    
    // It is required to call request() for a chunk of bytes. For a
    // fail safe implementation it is recommended to check that you
    // got the expected number of bytes (although the implementation
    // guarantees it).
    if (s.request(SIZE) < SIZE) throw new java.io.EOFException("request not successful"); 
 
    bufcnt = s.bufcnt;

    // whereas insert() has returned the incremented index,
    // extractInt() returns the value of the instance
    // variable. Therefore the index must be advanced manually. Please
    // note, that different types of instance variables require
    // different extract routines.
    d0 = java.io.UtilsIO.extractInt  (buf, bufcnt); bufcnt += 4;
    d1 = java.io.UtilsIO.extractFloat(buf, bufcnt); bufcnt += 4;
    d2 = java.io.UtilsIO.extractChar (buf, bufcnt); bufcnt += 2;
    d3 = java.io.UtilsIO.extractShort(buf, bufcnt);

    // Alternatively, if the absolute positions in the buffer are
    // known, it might be faster to do the extraction as follows:
    /*
    d0 = java.io.UtilsIO.extractInt  (buf, bufcnt     ); 
    d1 = java.io.UtilsIO.extractFloat(buf, bufcnt +  4); 
    d2 = java.io.UtilsIO.extractChar (buf, bufcnt +  8); 
    d3 = java.io.UtilsIO.extractShort(buf, bufcnt + 10);
    */

    // finally accept SIZE to prepare the buffer for the next chunk.
    s.accept(SIZE);
  }
}
